package ethz.ch.pp.seq;

import java.util.concurrent.ForkJoinPool;
import java.util.concurrent.RecursiveTask;


public class LongestCommonSequenceMultiOpt extends RecursiveTask<ReturnInfo> {
	
	private static final long serialVersionUID = 4179716026313995745L;
		
	private int[] input;
	private int start;
	private int length;
	
	private LongestCommonSequenceMultiOpt(int[] input, int start, int length) {		
		this.input = input;
		this.start = start;
		this.length = length;
	}
	
	public static Sequence longestCommonSequence(int[] input, int numThreads) {
		ForkJoinPool fjp = new ForkJoinPool(numThreads);
		ReturnInfo result = fjp.invoke(new LongestCommonSequenceMultiOpt(input, 0, input.length));	
		fjp.shutdown();
		return result.max;
	}

	@Override
	protected ReturnInfo compute() {
		if (this.length <= 100) {
			return LongestCommonSequence.longestCommonSequences(this.input, this.start, this.length);
		} else {			
			int halfSize = (this.length) / 2;

			LongestCommonSequenceMultiOpt firstHalf = 
					new LongestCommonSequenceMultiOpt(this.input, this.start, halfSize);
			
			LongestCommonSequenceMultiOpt secondHalf = 
					new LongestCommonSequenceMultiOpt(this.input, 
							this.start + halfSize, this.length - halfSize);						
			
			firstHalf.fork();
			ReturnInfo second = secondHalf.compute();
			ReturnInfo first = firstHalf.join();
			
			if (this.input[this.start + halfSize - 1] != this.input[this.start + halfSize]) {				
				return first.takeGreater(second);
			} else {
				/* If the end and beginning of the partitions are the same then 
				 * there could have been a sequence that spans both
				 * 
				 * Here, instead of computing the sequence from scratch, we define a way
				 * to compute the sequences in a way that allows them to be merged easily.
				 */
				return first.merge(second);
			}
			
			
		}
	}
}
