package ethz.ch.pp.seq;

/**
 * @author TA
 *
 * ReturnInfo aggregates three 
 * sequences returned by longestCommonSequences method.
 * First sequence is the sequence of the first element
 * in the array.
 * Max sequence is the largest sequence 
 * in the array. 
 * Last sequence is the sequence of the last element
 * in the array.
 */
public class ReturnInfo {
	public final Sequence first;
	public final Sequence max;
	public final Sequence last;
	
	public ReturnInfo(Sequence f, Sequence m, Sequence l) {
		first=f;
		max=m;
		last=l;
	}

	/**
	 * Creates a new return info from this and other
	 * taking as max the larger max sequence from the two 
	 * @param other - the other ReturnInfo
	 * @return ReturnInfo object with the maximum max field 
	 * taken from the two objects. Field first is taken from 
	 * this and field last from other.   
	 */
	public ReturnInfo takeGreater(ReturnInfo other) {
		Sequence[] list = {max, other.max}; 
		return new ReturnInfo(first, Sequence.max(list), other.last);
	}
	
	/**
	 * Merges two ReturnInfo objects (this and other). Field first is taken from 
	 * this and field last from other. Max is taken to be the largest sequence among 
	 * the max fields from the two object and the merged sequence this.last.merge(other.first)
	 * @param other - the other ReturnInfo
	 * @return
	 */
	public ReturnInfo merge(ReturnInfo other) {
		Sequence[] list = {max, other.max, last.merge(other.first)}; 
		return new ReturnInfo(first, Sequence.max(list), other.last);
	}
	
	@Override
	public String toString() {
		return "First " + first + "\n" +
			   "Max   " + max + "\n" +
			   "Last  " + last;
	}

}
