package ethz.ch.pp.seq;

public class Sequence {
	public final int startIndex;
	public final int endIndex;
	
	public Sequence(int start, int end) {
		startIndex = start;
		endIndex = end;
	}
	
	int lenght() {
		return endIndex - startIndex + 1;
	}	

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + endIndex;
		result = prime * result + startIndex;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof Sequence))
			return false;
		Sequence other = (Sequence) obj;
		return endIndex == other.endIndex && startIndex == other.startIndex;
	}

	@Override
	public String toString() {
		return "Sequence [start=" + startIndex + ", end=" + endIndex + "]";
	}
	
	boolean isGreaterThan(Sequence other) {
	    return (lenght() > other.lenght()) || (lenght() == other.lenght() && startIndex < other.startIndex);
	}
	
	/**
	 * Merges two adjacent sequences
	 * E.g., [4,5] [6,14] -> [4,14]
	 * @param other - the other sequence
	 * @return the merged sequence
	 */
	Sequence merge(Sequence other) {
		if(endIndex+1==other.startIndex) {
			return new Sequence(startIndex, other.endIndex);
		} else 
			throw new IllegalArgumentException("non adjecent sequences");
			
	}
	
	/**
	 * Returns the largest sequence from the list
	 * @param list - list of sequences
	 * @return largest sequence
	 */
	public static Sequence max(Sequence[] list) {
		Sequence ret = new Sequence(0, 0);
		for (Sequence s : list) {
			if (s.isGreaterThan(ret)) {
				ret = s;
			}
		}
		return ret;
	}
	
}
