package ethz.ch.pp.util;

public class ArrayUtils {

	/**
	 * Merges two elements from two ordered array into a single ordered array. 
	 * That is, output = sorted(in1 + in2)
	 *
	 * @param  in1  ordered array
	 * @param  in2  ordered array
	 * @param  output array to write ordered elements from in1 and in2 
	 */
	public static void merge(int[] in1, int[] in2, int[] output) {	

		int in1Position = 0;
		int in2Position = 0;
		int outputPosition = 0;

		while (in1Position < in1.length && in2Position < in2.length) {
			if (in1[in1Position] < in2[in2Position]) {
				output[outputPosition++] = in1[in1Position++];
			} else {
				output[outputPosition++] = in2[in2Position++];
			}
		}

		while (in1Position < in1.length) {
			output[outputPosition++] = in1[in1Position++];
		}

		while (in2Position < in2.length) {
			output[outputPosition++] = in2[in2Position++];
		}
	}
	
	/**
	 * Merges two ordered arrays A1 and A2 continuously stored in the array {@code in} 
	 * A1 starts at s1 and ends at s2-1
	 * A2 starts at s2 and ends at e2-1
	 * into an ordered array B.
	 * 
	 * The resulting array B is written in {@code out}.
	 * 
	 * Assumption is that the arrays are stored continuously
	 * and arrays {@code in} and {@code out} are defined between 
	 * indexes s1 and e2.
	 *  
	 * @param s1 - start of A1
	 * @param s2 - start of A2 (also end of A1 excluded)
	 * @param e2 - end of A2 (excluded)
	 * @param in - input array containing ordered arrays A1 and A2
	 * @param out - output array to be  populated with the ordered merged array B 
	 */
	public static void merge(int s1, int s2, int e2, int[] in, int[] out) {
		int e1=s2;

		int i1=s1;
		int i2=s2;
		
		int o = s1;
		
		while (i1 < e1 && i2 < e2) {
			if (in[i1] < in[i2]) {
				out[o++] = in[i1++];
			} else {
				out[o++] = in[i2++];
			}
		}
		
		while (i1 < e1) {
			out[o++] = in[i1++];
		}

		while (i2 < e2) {
			out[o++] = in[i2++];
		}
	}
}
